/*
 * File:   main.c
 * Author: muth
 * Purpose: LCD Clock
 * Created on February 13, 2016, 12:59 PM
 */
#include <xc.h>

// PIC18F87K90 Configuration Bit Settings
// CONFIG1L
#pragma config RETEN = ON       // VREG Sleep Enable bit (Enabled)
#pragma config INTOSCSEL = LOW // LF-INTOSC Low-power Enable bit (LF-INTOSC in High-power mode during Sleep)
#pragma config SOSCSEL = LOW   // SOSC Power Selection and mode Configuration bits (High Power SOSC circuit selected)
#pragma config XINST = OFF      // Extended Instruction Set (Disable)

// CONFIG1H
#pragma config FOSC = INTIO2    // Oscillator (Internal RC oscillator)
#pragma config PLLCFG = OFF     // PLL x4 Enable bit (Disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor (Disabled)
#pragma config IESO = OFF       // Internal External Oscillator Switch Over Mode (Disabled)

// CONFIG2L
#pragma config PWRTEN = OFF     // Power Up Timer (Disabled)
#pragma config BOREN = OFF      // Brown Out Detect (Disabled in hardware, SBOREN disabled)
#pragma config BORV = 3         // Brown-out Reset Voltage bits (1.8V)
#pragma config BORPWR = ZPBORMV // BORMV Power level (ZPBORMV instead of BORMV is selected)

// CONFIG2H
#pragma config WDTEN = OFF      // Watchdog Timer (WDT disabled in hardware; SWDTEN bit disabled)
#pragma config WDTPS = 1048576  // Watchdog Postscaler (1:1048576)

// CONFIG3L
#pragma config RTCOSC = SOSCREF // RTCC Clock Select (RTCC uses SOSC)

// CONFIG3H
#pragma config CCP2MX = PORTC   // CCP2 Mux (RC1)
#pragma config ECCPMX = PORTE   // ECCP Mux (Enhanced CCP1/3 [P1B/P1C/P3B/P3C] muxed with RE6/RE5/RE4/RE3)
#pragma config MSSPMSK = MSK7   // MSSP address masking (7 Bit address masking mode)
#pragma config MCLRE = OFF      // Master Clear Enable (MCLR Disabled, RG5 Enabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Overflow Reset (Enabled)
#pragma config BBSIZ = BB2K     // Boot Block Size (2K word Boot Block size)

// CONFIG5L
#pragma config CP0 = OFF        // Code Protect 00800-03FFF (Disabled)
#pragma config CP1 = OFF        // Code Protect 04000-07FFF (Disabled)
#pragma config CP2 = OFF        // Code Protect 08000-0BFFF (Disabled)
#pragma config CP3 = OFF        // Code Protect 0C000-0FFFF (Disabled)
#pragma config CP4 = OFF        // Code Protect 10000-13FFF (Disabled)
#pragma config CP5 = OFF        // Code Protect 14000-17FFF (Disabled)
#pragma config CP6 = OFF        // Code Protect 18000-1BFFF (Disabled)
#pragma config CP7 = OFF        // Code Protect 1C000-1FFFF (Disabled)

// CONFIG5H
#pragma config CPB = OFF        // Code Protect Boot (Disabled)
#pragma config CPD = OFF        // Data EE Read Protect (Disabled)

// CONFIG6L
#pragma config WRT0 = OFF       // Table Write Protect 00800-03FFF (Disabled)
#pragma config WRT1 = OFF       // Table Write Protect 04000-07FFF (Disabled)
#pragma config WRT2 = OFF       // Table Write Protect 08000-0BFFF (Disabled)
#pragma config WRT3 = OFF       // Table Write Protect 0C000-0FFFF (Disabled)
#pragma config WRT4 = OFF       // Table Write Protect 10000-13FFF (Disabled)
#pragma config WRT5 = OFF       // Table Write Protect 14000-17FFF (Disabled)
#pragma config WRT6 = OFF       // Table Write Protect 18000-1BFFF (Disabled)
#pragma config WRT7 = OFF       // Table Write Protect 1C000-1FFFF (Disabled)

// CONFIG6H
#pragma config WRTC = OFF       // Config. Write Protect (Disabled)
#pragma config WRTB = OFF       // Table Write Protect Boot (Disabled)
#pragma config WRTD = OFF       // Data EE Write Protect (Disabled)

// CONFIG7L
#pragma config EBRT0 = OFF      // Table Read Protect 00800-03FFF (Disabled)
#pragma config EBRT1 = OFF      // Table Read Protect 04000-07FFF (Disabled)
#pragma config EBRT2 = OFF      // Table Read Protect 08000-0BFFF (Disabled)
#pragma config EBRT3 = OFF      // Table Read Protect 0C000-0FFFF (Disabled)
#pragma config EBRT4 = OFF      // Table Read Protect 10000-13FFF (Disabled)
#pragma config EBRT5 = OFF      // Table Read Protect 14000-17FFF (Disabled)
#pragma config EBRT6 = OFF      // Table Read Protect 18000-1BFFF (Disabled)
#pragma config EBRT7 = OFF      // Table Read Protect 1C000-1FFFF (Disabled)

// CONFIG7H
#pragma config EBRTB = OFF      // Table Read Protect Boot (Disabled)

#define _XTAL_FREQ (8000000UL)

const unsigned char LCDdigit[10] = {0x7E, 0x0C, 0xB6, 0x9E, 0xCC, 0xDA, 0xF8,
                                    0x4E, 0xFE, 0xCE};
BOOL dots = 0;

// proto
void setTime(void);

void interrupt isr() {
    // RTC iterrupt
    if (PIR3bits.RTCCIF){
        /* */
        setTime();
        PIR3bits.RTCCIF = 0;
        SLEEP();
        NOP();
    }
}

void main(void) {
    // osc conf
    OSCCONbits.IDLEN = 1;
    OSCCONbits.SCS = 0b01;          // SOSC on idle

    // unlock RTC sequence
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;

    PIE3bits.RTCCIE = 1;
    PADCFG1bits.RTSECSEL = 0b01;
    RTCCFGbits.RTCEN = 1;

    ALRMCFGbits.ALRMEN = 1;
    ALRMCFGbits.CHIME = 1;
    ALRMCFGbits.AMASK = 0b0001;

    // LCD setup
    LCDSE0 = 0xFF;
    LCDSE1 = 0xFF;
    LCDSE2 = 0xFF;
    LCDSE3 = 0xFF;
    LCDSE4 = 0xFE;
    LCDSE5 = 0xFF;

    LCDCONbits.LCDEN = 1;
    LCDCONbits.SLPEN = 1;       // enable during sleep
    LCDCONbits.CS = 0b01;       // clock source
    LCDPSbits.LP = 0b0100;
    LCDREFbits.LCDIRE = 1;
    LCDREFbits.LCDCST = 0b000;
    LCDRLbits.LRLAP = 0b11;
    LCDRLbits.LRLAT = 0b111;

    // sleep conf perif
    PMD3 = 0xFF;
    PMD2 = 0xFF;
    PMD1 = 0b11011111;
    PMD0 = 0xFF;

    // General
    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1;

// time set
//    RTCCFGbits.RTCPTR = 0b00;
//    RTCVALHbits.RTCVALH = 0x33;
//    RTCCFGbits.RTCPTR = 0b01;
//    RTCVALLbits.RTCVALL = 0x15;

    while (1) {
        NOP();
    }
    return;
}

void setTime(void) {
    // dots
//    dots = RTCCFGbits.HALFSEC_bit;
    dots = !dots;

    //seconds
    RTCCFGbits.RTCPTR = 0b00;
    LCDDATA4 = LCDdigit[RTCVALLbits.RTCVALL & 0x0F];
    LCDDATA5 = LCDdigit[(RTCVALLbits.RTCVALL & 0xF0)>>4]+ dots;

    //minutes
    LCDDATA0 = LCDdigit[RTCVALHbits.RTCVALH & 0x0F]+ dots;
    LCDDATA1 = LCDdigit[(RTCVALHbits.RTCVALH & 0xF0)>>4]+ dots;

    //hours
    RTCCFGbits.RTCPTR = 0b01;
    LCDDATA2 = LCDdigit[RTCVALLbits.RTCVALL & 0x0F]+ dots;
    LCDDATA3 = LCDdigit[(RTCVALLbits.RTCVALL & 0xF0)>>4];

}

